#!/bin/sh /usr/share/centrifydc/perl/run

# Copyright (C) 2009-2020 Centrify Corporation. All rights reserved.
#
# Machine mapper script
#

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;
use CentrifyDC::GP::Args;
use CentrifyDC::GP::General qw(:debug CreateDir);
use CentrifyDC::GP::Lock;
use CentrifyDC::GP::Registry;
use CentrifyDC::GP::RegHelper;
use CentrifyDC::GP::General qw(RunCmd GetTempDirPath EscapeSingleQuote);
use CentrifyDC::GP::CimsPath qw($CIMS_USR $CIMS_INSTALL);
use CentrifyDC::GP::GPIsolation qw(GetRegKey);
use CentrifyDC::GP::CertUtil qw(GetFingerprint IsExpired);
use CentrifyDC::GP::Conf qw(GetValueList);

use File::Glob qw(:glob csh_glob);
use File::Basename;


my $tempdir = GetTempDirPath(0);
defined($tempdir) or FATAL_OUT();


my $adcert = "$CIMS_INSTALL/sbin/adcert";
my $openssl = "$CIMS_INSTALL/bin/openssl";
my $curl = "$CIMS_INSTALL/bin/curl";
my $ldapsearch = "$CIMS_INSTALL/bin/ldapsearch";
my $c_rehash = "$CIMS_INSTALL/bin/c_rehash";

# c_rehash needs this environment variable for openssl binary
$ENV{OPENSSL} = $openssl;

my $enrollreg = GetRegKey("AutoEnroll");
# list of registry keys to get cert
my @certRootKeys = (
'Software/Policies/Microsoft/SystemCertificates/Root/Certificates',
'Software/Policies/Microsoft/SystemCertificates/CA/Certificates',
);
my $certcontainers = $CentrifyDC::Config::properties{"adsec.cert.containers"};
my $ret;
my $hash;
my $out;
my $count;

my $CERTSTORE;

if (!defined($certcontainers))
{
    $certcontainers = "AIA,Certification Authorities,Enrollment Services,NTAuthCertificates";
}

# create a new Args for a mapper
my $args = CentrifyDC::GP::Args->new('machine');

CentrifyDC::GP::Registry::Load(undef);

$CERTSTORE = CentrifyDC::Config::GetCertStore(undef);

if ($args->isMap())
{
    # make sure download dir exists
    CreateDir(${CERTSTORE});
    
    if ($args->class() eq "machine")
    {
        # step 1 make sure local copy of policy defined trust certs are up to date.
        # we might need to get certs from other places too
        #
        my @rootCerts = csh_glob("${CERTSTORE}/trust_*.cert");
        my $connectstate = `adinfo -m`;
        chomp($connectstate);
        if ($connectstate eq 'connected')
        {
            unlink @rootCerts;
        }
        
        my @certKeys = ();
        foreach my $rootKey (@certRootKeys)
        {
            push(@certKeys, CentrifyDC::GP::Registry::GetSubKeys($rootKey, "current", undef));
        }
        
        my $count = @certKeys;
        if ($count > 0)
        {
            DEBUG_OUT("Found $count certs in registry");
            foreach my $cname (@certKeys)
            {
                DEBUG_OUT("Found cert under registry key [$cname]");
            }
        }
        
        foreach my $cname (@certKeys)
        {
            DEBUG_OUT("Get cert under registry key [$cname]");
            # blob contains the cert info
            my @ckey = CentrifyDC::GP::Registry::Query("machine", $cname, "current", "blob");
            my $cert = DecodeCertBlob($ckey[1]);
            
            if (defined($cert))
            {
                # write temp copy in der format
                open CERT, ">${CERTSTORE}/temp.der";
                binmode CERT;
                print CERT $cert;
                close CERT;
                InstallCert("${CERTSTORE}/temp.der");
                unlink "${CERTSTORE}/temp.der";
            }
        }
        
        # now fetch root certs from configuration containers in AD
        foreach my $certcontainer (split(",", $certcontainers))
        {
            GetAdCerts($certcontainer);
        }
    }

    # install additional certs
    my $prop_name = 'gp.mappers.certgp.pl.additional.cafiles';
    my @entries = CentrifyDC::GP::Conf::GetValueList($prop_name);
    foreach my $entry (@entries)
    {
        InstallCert($entry);
    }
    
    my $reg = CentrifyDC::GP::RegHelper->new('map', $args->class(), $enrollreg, "aepolicy", undef);
    $reg or return undef;
    $reg->load();
    
    my $autoPolicy = $reg->get("current");
    # now see if we need to do machine auto enrollment. Refer to GP setting:
    # Computer Configuration->Policies->Windows Settings->Security Settings->Public Key Policies
    #    ->Certificate Services Client-Auto Enrollment->Configuration Model
    # If the policy is "Enabled", we need to do autoenrollment. 
    # If the policy is "Not configured" or "Disabled", we don't need to do autoenrollment. 
    # The disable bit is 0x8000 which is equal to 32768 in decimal. 
    # NOTE: Win2k3(R2) enables this policy by default(there's no "Not configured" for this GP setting). 
    # So CDC enabled auto-enrollment by default before. Now the behavior has been changed and follows Windows2k8 
    if(defined($autoPolicy) && $autoPolicy != 32768)
    {
        # we dont care about the values in the policy
        # maybe we should take the flags and pass to adcert, but we dont
        
        my $aeCmd = [$adcert, "-d", $CERTSTORE, "-m"];
        
        my $system = `uname -s`;
        chomp($system);
        if ($system eq 'Darwin')
        {
            my $useKeychain = (CentrifyDC::GP::Registry::Query("machine", "Software/Policies/Centrify/CentrifyDC/Settings/Mac/Security/PublicKeyPolicies", "current", "UseKeychain"))[1];
            if (defined($useKeychain) && $useKeychain eq "YES")
            {
                push(@$aeCmd, "-k");
                my $privateKeyExtractable = (CentrifyDC::GP::Registry::Query("machine", "Software/Policies/Centrify/CentrifyDC/Settings/Mac/Security/PublicKeyPolicies", "current", "PrivateKeyExtractable"))[1];
                if (defined($privateKeyExtractable) && $privateKeyExtractable eq "NO")
                {
                    push(@$aeCmd, "-X");
                }
            }

            my $PUBLICKEY_POLICIES_KEY = 'Software/Policies/Centrify/CentrifyDC/Settings/Mac/Security/PublicKeyPolicies';
            my $PRIVATEKEY_APP_ACL = $PUBLICKEY_POLICIES_KEY . "/PrivateKeyAppACL";
            my $allowAllAppAccess = (CentrifyDC::GP::Registry::Query("machine", $PUBLICKEY_POLICIES_KEY, "current", "AllowAllApplicationAccessPrivateKey"))[1];
            if (defined($allowAllAppAccess) && $allowAllAppAccess eq "YES") 
            {
                DEBUG_OUT("Allowing all apps access to private key.");
                push(@$aeCmd, "-E");
            } 
            else 
            {
                push(@$aeCmd, "-a");
                push(@$aeCmd, "$adcert");
                my @reg_values = CentrifyDC::GP::Registry::Values("machine", $PRIVATEKEY_APP_ACL, "current");
                # Remove empty value from array
                @reg_values = grep /\S/, @reg_values;

                my $len = scalar(@reg_values);
                if ($len > 0) {
                    DEBUG_OUT("Allowing the following applications: ");
                    foreach my $reg_value (@reg_values)
                    {
                        my ($reg_type, $reg_data) = CentrifyDC::GP::Registry::Query("machine", $PRIVATEKEY_APP_ACL, "current", $reg_value);
                        if (defined($reg_data))
                        {
                            DEBUG_OUT("\t$reg_data");
                            push(@$aeCmd, "-a");
                            push(@$aeCmd, "$reg_data");
                        }
                    }
                    DEBUG_OUT("to access private key");
                }
            }
        }
        
        my ($rc, $out) = RunCmd($aeCmd);
        
        # now get crls for the certs we just got
        my @certFiles = csh_glob("${CERTSTORE}/*.cert");
        foreach my $certFile (@certFiles)
        {
            DEBUG_OUT("Fetch CRL for cert $certFile");
            RunCmd(["$CIMS_INSTALL/sbin/get_crl.pl", $certFile, "$tempdir/crl", "1"]);
        }
    }
    
    my @fetchfiles = csh_glob("$tempdir/asynch_crl_fetch_*.crl");
    foreach my $fetchfile (@fetchfiles)
    {
        RunCmd(["$CIMS_INSTALL/sbin/get_crl.pl", $fetchfile, "$tempdir/crl", "1"]);
        unlink $fetchfile;
    }
    
    # Do a rehash to make sure all certs can be used.
    RehashAllCerts();
}
else
{
    # remove existing certificate information but keep the auto-enrolled
    # cert and private key.
    my @certItems = csh_glob("${CERTSTORE}/*");
    my @deleteItems;
    foreach my $item (@certItems)
    {
        if ($item !~ m/^${CERTSTORE}\/auto_.*/ && $item !~ m/^${CERTSTORE}\/autoeth_.*/)
        {
            push(@deleteItems, $item);
        }
    }
    unlink(@deleteItems);
}

# read a cert from the registry
# it is laid out as follows (found by debuggin windows code)
# <type><x><length><value>
# type,x,value are all little endian 32 bit integers
# I suspect x is an 'occurs' count. I have only ever seen 1 in it
# Length is the length of the value in bytes
# I dont know or care what the values are except for type=32 which is the cert DER itself
# the input is a hexified string
sub DecodeCertBlob
{
    (my $blob) = @_;
    # first convert the hexified string to the binary it represents
    my $blen = length $blob;
    my $bin = pack("H${blen}", $blob);
    
    my $off = 0;
    while($off < length $bin)
    {
        (my $type, my $x, my $len) = unpack("VVV", substr($bin, $off));
        $off += 12;
        if($type == 32)
        {
            return substr($bin, $off, $len);
        }
        $off += $len;
    }
    return undef;
}

# Retrieve [root] certificates from a specified AD container. ldapsearch is
# used to retrieve the domain name, then the specified container is searched by
# looking for the "cacertificate" attribute (which is where certificates will
# be stored). Those found are considered trusted root certificates, and are
# converted from DER to PEM format and stored as "trust_<fingerprint>.cert".
#
sub GetAdCerts
{
    (my $container) = @_;
    
    my $val;
    my $dcdom;
    
    ($ret, $out) = RunCmd([$ldapsearch, "-m", "-LLL", "-s", "base", "-b", "", "-r", "configurationNamingContext"]);
    foreach $val (split /\n/, $out)
    {
        if ($val =~ /^configurationNamingContext: (.*)$/)
        {
            $dcdom = $1;
            last;
        }
    }
    my $ldapcmd = [$ldapsearch, "-m", "-LLL", "-t", "-b", "CN=${container},CN=Public Key Services,CN=Services,${dcdom}", "-r", "cacertificate"];
    ($ret, $out) = RunCmd($ldapcmd);
    my @ldap = split /\n/, $out;
    my @cacerts = ();
    foreach $val (@ldap)
    {
        if ($val =~ /^cACertificate:< file:\/\/(.*)/)
        {
            push @cacerts, $1;
        }
    }
    
    foreach my $temp (@cacerts)
    {
        InstallCert($temp);
        unlink ${temp};
    }
    
    # If we find any cert then we should do a rehash so that the cert
    # can be used by other tools, like adcert.
    if (@cacerts > 0)
    {
        RehashAllCerts();
    }
}

#
# Install certificate to CERTSTORE folder which is /var/centrify/net/certs by
# default.
#
#   $_[0]:  file-based certificate input
#   $_[1]:  file-based certificate input format: PEM/DER/NET
#
#   return: 1       - successful
#           undef   - failed
#
sub InstallCert($;$)
{
    my ($cert, $certForm) = @_;
    
    if ($certForm ne 'PEM' && $certForm ne 'DER' && $certForm ne 'NET')
    {
        $certForm = 'DER';
    }
    
    # If the certificate is expired, skip installation.
    my $expired = IsExpired($cert, $certForm);
    if (defined($expired) && $expired)
    {
        DEBUG_OUT("Did not install cert $cert, because it's expired.");
        return undef;
    }    

    # we will use the fingerprint as the cert name
    my $ret = undef;
    my $fingerprint = GetFingerprint($cert, $certForm);
    if (defined($fingerprint))
    {
        chomp $fingerprint;
        # Don't install this cert file, if its fingerprint is excluded.
        my $prop_name = 'gp.mappers.certgp.pl.exclude.cacerts';
        my @entries = CentrifyDC::GP::Conf::GetValueList($prop_name);
        foreach my $entry (@entries)
        {
            if ($entry eq $fingerprint)
            {
                DEBUG_OUT("Did not install cert $cert, because its fingerprint($fingerprint) is excluded.");
                return undef;
            }
        }

        # now convert to PEM format
        my ($rc, $output) = RunCmd([$openssl, "x509", "-in", "$cert", "-inform", "$certForm", "-out", "${CERTSTORE}/trust_${fingerprint}.cert"]);
        if ($rc == 0)
        {
            chmod(0444, "${CERTSTORE}/trust_${fingerprint}.cert");
            $ret = 1;
        }
        else
        {
            DEBUG_OUT("Can not convert $cert to ${CERTSTORE}/trust_${fingerprint}.cert: $output");
        }
    }
    else
    {
        # normally the return output is the failure reason
        DEBUG_OUT("Can not get fingerprint from $cert: $fingerprint");
    }
    return $ret;
}

# Rehash all certificates
# The openssl will look for certificate with the filename <subject-hash>.n.
# This is a convention to call c_rehash (a Perl script from openssl) to
# recreate symlink <subject-hash>.n for all the certifiicate files.
#
sub RehashAllCerts
{
    my $rc = RunCmd([$c_rehash, $CERTSTORE]);
    if (!defined($rc) or $rc ne '0')
    {
        WARN_OUT("Cannot rehash symlink in $CERTSTORE")
    }
}
